/********************************************************************
 FileName:		main.c
 Dependencies:	See INCLUDES section
 Processor:		PIC18, PIC24, and PIC32 USB Microcontrollers
 Hardware:		This demo is natively intended to be used on Microchip USB demo
 				boards supported by the MCHPFSUSB stack.  See release notes for
 				support matrix.  This demo can be modified for use on other hardware
 				platforms.
 Complier:  	Microchip C18 (for PIC18), C30 (for PIC24), C32 (for PIC32)
 Company:		Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the companyE for its PIC Microcontroller is intended and
 supplied to you, the Companys customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN AS ISECONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
********************************************************************/

#ifndef IRREMOCONMAIN_C
#define IRREMOCONMAIN_C

/** INCLUDES *******************************************************/
#include <adc.h>
#include <timers.h>
#include <pwm.h>
#include <string.h>
#include "./USB/usb.h"
#include "HardwareProfile.h"
#include "./USB/usb_function_hid.h"

/** CONFIGURATION **************************************************/
#pragma config CPUDIV = NOCLKDIV
#pragma config USBDIV = OFF
#pragma config FOSC   = HS
#pragma config PLLEN  = ON
#pragma config FCMEN  = OFF
#pragma config IESO   = OFF
#pragma config PWRTEN = OFF
#pragma config BOREN  = OFF
#pragma config BORV   = 30
#pragma config WDTEN  = OFF
#pragma config WDTPS  = 32768
#pragma config MCLRE  = OFF
#pragma config HFOFST = OFF
#pragma config STVREN = ON
#pragma config LVP    = OFF
#pragma config XINST  = OFF
#pragma config BBSIZ  = OFF
#pragma config CP0    = OFF
#pragma config CP1    = OFF
#pragma config CPB    = OFF
#pragma config WRT0   = OFF
#pragma config WRT1   = OFF
#pragma config WRTB   = OFF
#pragma config WRTC   = OFF
#pragma config EBTR0  = OFF
#pragma config EBTR1  = OFF
#pragma config EBTRB  = OFF


/** PRIVATE PROTOTYPES *********************************************/
static void InitializeSystem(void);
void ProcessIO(void);
void UserInit(void);
void YourHighPriorityISRCode();
void YourLowPriorityISRCode();
int RemoconOutData(void);

/** DECLARATIONS ***************************************************/

#define TX_BUFFER_SIZE 64			/* USBMobt@TCY */
#define RX_BUFFER_SIZE 64			/* USBMobt@TCY */

#define NUM_OF_PINS		12

#define PIN1	PORTCbits.RC5
#define	PIN2	PORTCbits.RC4
#define	PIN3	PORTCbits.RC3
#define PIN4	PORTCbits.RC6
#define	PIN5	PORTCbits.RC7
#define	PIN6	PORTBbits.RB7
#define	PIN7	PORTBbits.RB6
#define	PIN8	PORTBbits.RB5
#define	PIN9	PORTBbits.RB4
#define	PIN10	PORTCbits.RC2
#define PIN11	PORTCbits.RC1
#define	PIN12	PORTCbits.RC0

/* Timer0 0.1ms */
/* 100us / ( 0.021us x 4 ) = 1200  48MHz = 1/48us = 0.021us */
/* 1200 / 8 = 150    vXP[=8*/
/* 0x10000 - 0x96 = 0xFF88    16bitJE^*/
#define WRITE_TIMER0_COUNT        0xFF6A        //Timer0̎

//38kHz=26us
// 38kHz = ( 78 + 1 ) * 4 * 0.0208us(48MHz) * 4(vXP[) = 26.2912us
#define PWM_PERIOD	0x4E 	/* Period 78 = 0x4E */
#define	PWM_OFF		0
#define PWM_50		0x9E	/* PWM Duty 50% = (Period + 1) * 4 * 0.5 */
#define PWM_100		0x13C	/* PWM Duty 100% = (Period + 1) * 4 * 1.0 */

#define OUTSTATUS_OUTPUT_WAIT			0x00	/* o͏ o͑҂ */
#define OUTSTATUS_READER_CODE			0x01	/* o͏ [_R[h */
#define OUTSTATUS_DATA_CODE				0x02	/* o͏ f[^R[h */
#define OUTSTATUS_END					0x04	/* o͏ I */
#define OUTBUFFER_READER_SIZE			1		/* o̓obt@TCY [_R[h(1byte) */
#define OUTBUFFER_DATA_SIZE				6		/* o̓obt@TCY f[^(6byte) */
#define OUTBUFFER_SIZE					OUTBUFFER_READER_SIZE+OUTBUFFER_DATA_SIZE	/* o̓obt@TCY [_R[h(1byte) + f[^(6byte) */
#define OUTBUFFER_FORMAT_SIZE_EX		1		/* g o̓obt@TCY tH[}bgR[h(1byte) */
#define OUTBUFFER_DATA_SIZE_EX			2		/* g o̓obt@TCY f[^TCYiPbitj(1byte) */
#define OUTBUFFER_DATA_AREA_SIZE_EX		32		/* g o̓obt@TCY f[^GATCY(32byte) */
#define OUTBUFFER_SIZE_EX				OUTBUFFER_FORMAT_SIZE_EX+OUTBUFFER_DATA_SIZE_EX+OUTBUFFER_DATA_AREA_SIZE_EX	/* g o̓obt@TCY [_R[h(1byte) + f[^(32byte) */

#define FORMATCODE_UNKNOWN				0		/* ԊOʐMtH[}bg s */
#define FORMATCODE_NEC					2		/* ԊOʐMtH[}bg NEC */

#define SENDCODE_END_CNT				200 	/* IR[h̉OFF𑱂 1000 = 100[ms] / 0.1[ms] */
#define DATA_MAX_BITS					0xFF	/* f[^R[h̍őrbg */

// PC-6601SR̐ԊO^C~Of[^
// ɂiRpC̍œKˑ?ĵŔB͖̂Ȃ݂
#define READERCODE_ON_NEC				22		// [_R[h ON
#define READERCODE_OFF_NEC				4		// [_R[h OFF
#define DATA0_ON_NEC					4		// f[^0 ON
#define DATA0_OFF_NEC					2		// f[^0 OFF
#define DATA1_ON_NEC					4		// f[^1 ON
#define DATA1_OFF_NEC					9		// f[^1 OFF

#define CODE_TYPE_STANDARD	0	// R[h^Cv W

/** VARIABLES ******************************************************/
#pragma udata
char c_version[]="6601SR_1.00";

BYTE mouse_buffer[4] = {0};
BYTE volume_buffer[1] = {0};
BYTE keyboard_buffer[8];
USB_HANDLE lastTransmission;
USB_HANDLE lastTransmission2;
USB_HANDLE lastINTransmissionKeyboard;
USB_HANDLE lastOUTTransmissionKeyboard;
USB_HANDLE USBOutHandle = 0;
USB_HANDLE USBInHandle = 0;

unsigned char uc_format_type = FORMATCODE_UNKNOWN;

// ԊORf[^Mpϐ
unsigned int ui_out_pos = 0;
unsigned char uc_out_status = OUTSTATUS_OUTPUT_WAIT;
unsigned char uc_out_buff[OUTBUFFER_SIZE_EX];
unsigned int ui_out_on_time = 0;
unsigned int ui_out_off_time = 0;
unsigned char uc_out_data_len = 0;
unsigned char uc_out_data0_on = 0;
unsigned char uc_out_data0_off = 0;
unsigned char uc_out_data1_on = 0;
unsigned char uc_out_data1_off = 0;
unsigned char uc_send_bit_size = 0;

unsigned char uc_out_code_type = CODE_TYPE_STANDARD;
unsigned int ui_PWM50_Set_Val = PWM_50;

// usbł̑MɎgobt@͂Ő錾
#pragma udata usbram2

BYTE volume_input[1];
BYTE mouse_input[4];
BYTE hid_report[8];
unsigned char volume_input_out_flag = 0;
unsigned char mouse_input_out_flag = 0;
unsigned char hid_report_out_flag = 0;
unsigned char ReceivedDataBuffer[RX_BUFFER_SIZE];
unsigned char ToSendDataBuffer[TX_BUFFER_SIZE];

#pragma udata

/** VECTOR REMAPPING ***********************************************/
#if defined(__18CXX)
	//On PIC18 devices, addresses 0x00, 0x08, and 0x18 are used for
	//the reset, high priority interrupt, and low priority interrupt
	//vectors.  However, the current Microchip USB bootloader
	//examples are intended to occupy addresses 0x00-0x7FF or
	//0x00-0xFFF depending on which bootloader is used.  Therefore,
	//the bootloader code remaps these vectors to new locations
	//as indicated below.  This remapping is only necessary if you
	//wish to program the hex file generated from this project with
	//the USB bootloader.  If no bootloader is used, edit the
	//usb_config.h file and comment out the following defines:
	//#define PROGRAMMABLE_WITH_USB_HID_BOOTLOADER
	//#define PROGRAMMABLE_WITH_USB_LEGACY_CUSTOM_CLASS_BOOTLOADER

	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
	#elif defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x800
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x808
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x818
	#else
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
	#endif

	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	extern void _startup (void);        // See c018i.c in your C18 compiler dir
	#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
	void _reset (void)
	{
	    _asm goto _startup _endasm
	}
	#endif
	#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
	void Remapped_High_ISR (void)
	{
	     _asm goto YourHighPriorityISRCode _endasm
	}
	#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
	void Remapped_Low_ISR (void)
	{
	     _asm goto YourLowPriorityISRCode _endasm
	}

	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	//Note: If this project is built while one of the bootloaders has
	//been defined, but then the output hex file is not programmed with
	//the bootloader, addresses 0x08 and 0x18 would end up programmed with 0xFFFF.
	//As a result, if an actual interrupt was enabled and occured, the PC would jump
	//to 0x08 (or 0x18) and would begin executing "0xFFFF" (unprogrammed space).  This
	//executes as nop instructions, but the PC would eventually reach the REMAPPED_RESET_VECTOR_ADDRESS
	//(0x1000 or 0x800, depending upon bootloader), and would execute the "goto _startup".  This
	//would effective reset the application.

	//To fix this situation, we should always deliberately place a
	//"goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS" at address 0x08, and a
	//"goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS" at address 0x18.  When the output
	//hex file of this project is programmed with the bootloader, these sections do not
	//get bootloaded (as they overlap the bootloader space).  If the output hex file is not
	//programmed using the bootloader, then the below goto instructions do get programmed,
	//and the hex file still works like normal.  The below section is only required to fix this
	//scenario.
	#pragma code HIGH_INTERRUPT_VECTOR = 0x08
	void High_ISR (void)
	{
	     _asm goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS _endasm
	}
	#pragma code LOW_INTERRUPT_VECTOR = 0x18
	void Low_ISR (void)
	{
	     _asm goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS _endasm
	}
	#endif	//end of "#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_LEGACY_CUSTOM_CLASS_BOOTLOADER)"

	#pragma code


	//These are your actual interrupt handling routines.
	#pragma interrupt YourHighPriorityISRCode
	void YourHighPriorityISRCode()
	{
		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.
		#if defined(USB_INTERRUPT)
	        USBDeviceTasks();
		#endif

	}	//This return will be a "retfie fast", since this is in a #pragma interrupt section
	#pragma interruptlow YourLowPriorityISRCode
	void YourLowPriorityISRCode()
	{
		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.

		if(INTCONbits.TMR0IF == 1)
		{
			INTCONbits.TMR0IF = 0;
			WriteTimer0(WRITE_TIMER0_COUNT);

			// f[^o͎͒̎MȂ
			if( uc_out_status == OUTSTATUS_OUTPUT_WAIT )
			{
			}

			/* ԊOR̃f[^o */
			RemoconOutData();
		}

	}	//This return will be a "retfie", since this is in a #pragma interruptlow section
#endif

#pragma code

/********************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *******************************************************************/
void main(void)
{
	InitializeSystem();

	#if defined(USB_INTERRUPT)
		USBDeviceAttach();
	#endif

//	^C}0̐ݒ
	OpenTimer0(TIMER_INT_ON & T0_16BIT & T0_SOURCE_INT & T0_PS_1_8);
	WriteTimer0(WRITE_TIMER0_COUNT);
//	CCPW[PWMݒ
	OpenPWM1(PWM_PERIOD);
//	^C}2̐ݒ vXP[1:4
	OpenTimer2(TIMER_INT_OFF & T2_PS_1_4 & T2_POST_1_1 );
	SetDCPWM1(PWM_OFF);
//	SetDCPWM1(PWM_50);

//	荞݊Jn
	RCONbits.IPEN = 1;	//荞ݗDt
	INTCON2bits.TMR0IP = 0;	//^C}0ʃx荞݂ɐݒ
	INTCONbits.TMR0IE = 1;	//^C}0荞݋
	INTCONbits.GIEL = 1;	//ʃx荞݂̋
//	INTCONbits.GIEH = 1;	//ʃx荞݂̋
	INTCONbits.GIE = 1;

	while(1)
	{
		#if defined(USB_POLLING)
		// Check bus status and service USB interrupts.
		USBDeviceTasks(); // Interrupt or polling method.  If using polling, must call
						  // this function periodically.  This function will take care
						  // of processing and responding to SETUP transactions
						  // (such as during the enumeration process when you first
						  // plug in).  USB hosts require that USB devices should accept
						  // and process SETUP packets in a timely fashion.  Therefore,
						  // when using polling, this function should be called
						  // frequently (such as once about every 100 microseconds) at any
						  // time that a SETUP packet might reasonably be expected to
						  // be sent by the host to your device.  In most cases, the
						  // USBDeviceTasks() function does not take very long to
						  // execute (~50 instruction cycles) before it returns.
		#endif


		// Application-specific tasks.
		// Application related code may be added here, or in the ProcessIO() function.
		ProcessIO();
	}//end while
}//end main


/********************************************************************
 * Function:        static void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        InitializeSystem is a centralize initialization
 *                  routine. All required USB initialization routines
 *                  are called from here.
 *
 *                  User application initialization routine should
 *                  also be called from here.
 *
 * Note:            None
 *******************************************************************/
static void InitializeSystem(void)
{
	ADCON1 |= 0x0F;                 // Default all pins to digital

//	The USB specifications require that USB peripheral devices must never source
//	current onto the Vbus pin.  Additionally, USB peripherals should not source
//	current on D+ or D- when the host/hub is not actively powering the Vbus line.
//	When designing a self powered (as opposed to bus powered) USB peripheral
//	device, the firmware should make sure not to turn on the USB module and D+
//	or D- pull up resistor unless Vbus is actively powered.  Therefore, the
//	firmware needs some means to detect when Vbus is being powered by the host.
//	A 5V tolerant I/O pin can be connected to Vbus (through a resistor), and
// 	can be used to detect when Vbus is high (host actively powering), or low
//	(host is shut down or otherwise not supplying power).  The USB firmware
// 	can then periodically poll this I/O pin to know when it is okay to turn on
//	the USB module/D+/D- pull up resistor.  When designing a purely bus powered
//	peripheral device, it is not possible to source current on D+ or D- when the
//	host is not actively providing power on Vbus. Therefore, implementing this
//	bus sense feature is optional.  This firmware can be made to use this bus
//	sense feature by making sure "USE_USB_BUS_SENSE_IO" has been defined in the
//	HardwareProfile.h file.
	#if defined(USE_USB_BUS_SENSE_IO)
	tris_usb_bus_sense = INPUT_PIN; // See HardwareProfile.h
	#endif

//	If the host PC sends a GetStatus (device) request, the firmware must respond
//	and let the host know if the USB peripheral device is currently bus powered
//	or self powered.  See chapter 9 in the official USB specifications for details
//	regarding this request.  If the peripheral device is capable of being both
//	self and bus powered, it should not return a hard coded value for this request.
//	Instead, firmware should check if it is currently self or bus powered, and
//	respond accordingly.  If the hardware has been configured like demonstrated
//	on the PICDEM FS USB Demo Board, an I/O pin can be polled to determine the
//	currently selected power source.  On the PICDEM FS USB Demo Board, "RA2"
//	is used for	this purpose.  If using this feature, make sure "USE_SELF_POWER_SENSE_IO"
//	has been defined in HardwareProfile.h, and that an appropriate I/O pin has been mapped
//	to it in HardwareProfile.h.
	#if defined(USE_SELF_POWER_SENSE_IO)
	tris_self_power = INPUT_PIN;	// See HardwareProfile.h
	#endif

	UserInit();

	USBDeviceInit();	//usb_device.c.  Initializes USB module SFRs and firmware
						//variables to known states.
}//end InitializeSystem



/******************************************************************************
 * Function:        void UserInit(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This routine should take care of all of the demo code
 *                  initialization that is required.
 *
 * Note:
 *
 *****************************************************************************/
void UserInit(void)
{
	int fi,fj;
	char mode_changedp = 1;

	TRISB = 0x00;	//RB5 b7:RB7 b6:RB6 b5:RB5 b4:RB4
	TRISC = 0x80;	//RCo  b7:RC7 b6:RC6 b5:RC5 b4:RC4 b3:RC3 b2:RC2 b1:RC1 bit0:RC0

	LATC = 0xff;

	ANSEL = 0x00;
	ANSELH = 0x00;	//SăfW^

	INTCON2bits.RABPU = 0;	//vAbvg悤ɂ
	WPUBbits.WPUB4 = 1;
	WPUBbits.WPUB5 = 1;
	WPUBbits.WPUB6 = 1;
	WPUBbits.WPUB7 = 1;

	//initialize the variable holding the handle for the last

	for(fi = 0; fi < OUTBUFFER_SIZE_EX; fi++ )
	{
	    uc_out_buff[fi] = 0;
	}

	// transmission
	lastTransmission = 0;
	lastTransmission2 = 0;
	lastINTransmissionKeyboard = 0;
	lastOUTTransmissionKeyboard = 0;
	USBOutHandle = 0;
	USBInHandle = 0;

	volume_input[0] = 0;

	mouse_input[0] =
	mouse_input[1] =
	mouse_input[2] = 0;

	hid_report_in[0] =
	hid_report_in[1] =
	hid_report_in[2] =
	hid_report_in[3] =
	hid_report_in[4] =
	hid_report_in[5] =
	hid_report_in[6] =
	hid_report_in[7] = 0;

	mouse_buffer[0] =
	mouse_buffer[1] =
	mouse_buffer[2] =
	mouse_buffer[3] = 0;

	keyboard_buffer[0] =
	keyboard_buffer[1] =
	keyboard_buffer[2] =
	keyboard_buffer[3] =
	keyboard_buffer[4] =
	keyboard_buffer[5] =
	keyboard_buffer[6] =
	keyboard_buffer[7] = 0;

	volume_buffer[0] = 0;

}//end UserInit


/********************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user
 *                  routines. It is a mixture of both USB and
 *                  non-USB tasks.
 *
 * Note:            None
 *******************************************************************/
void ProcessIO(void)
{
	int fi,fj;
	char result_button_press;
	char tmp;
	char result;
	unsigned char uc_1data = 0;

	// User Application USB tasks
	if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;

	//---------------------------------------------------------------------
	//	USBf[^M
	if(!HIDTxHandleBusy(lastTransmission))
	{
		//copy over the data to the HID buffer
		//}EXf[^̑M
		mouse_input[0] = mouse_buffer[0];
		mouse_input[1] = mouse_buffer[1];
		mouse_input[2] = mouse_buffer[2];
		mouse_input[3] = mouse_buffer[3];

		mouse_buffer[0] =0;
		mouse_buffer[1] =0;
		mouse_buffer[2] =0;
		mouse_buffer[3] =0;

		if( mouse_input_out_flag > 0 )
		{
			//Send the 8 byte packet over USB to the host.
			lastTransmission = HIDTxPacket(HID_EP, (BYTE*)&mouse_input, sizeof(mouse_input));
			mouse_input_out_flag--;
 		}
	}

	if(!HIDTxHandleBusy(lastINTransmissionKeyboard))
	{	       	//Load the HID buffer
		hid_report_in[0] = keyboard_buffer[0];
		hid_report_in[1] = keyboard_buffer[1];
		hid_report_in[2] = keyboard_buffer[2];
	    hid_report_in[3] = keyboard_buffer[3];
		hid_report_in[4] = keyboard_buffer[4];
		hid_report_in[5] = keyboard_buffer[5];
		hid_report_in[6] = keyboard_buffer[6];
		hid_report_in[7] = keyboard_buffer[7];

		keyboard_buffer[0] =
		keyboard_buffer[2] =
		keyboard_buffer[3] =
		keyboard_buffer[4] =
		keyboard_buffer[5] =
		keyboard_buffer[6] =
		keyboard_buffer[7] = 0;

		if( hid_report_out_flag > 0 )
		{
			//Send the 8 byte packet over USB to the host.
			lastINTransmissionKeyboard = HIDTxPacket(HID_EP3, (BYTE*)hid_report_in, 0x08);
			hid_report_out_flag--;
		}
	}

	if(!HIDTxHandleBusy(lastTransmission2))
	{
		volume_input[0] = volume_buffer[0];
		volume_buffer[0] = 0;

		if( volume_input_out_flag > 0 )
		{
			lastTransmission2 = HIDTxPacket(HID_EP2, (BYTE*)&volume_input, sizeof(volume_input));
			volume_input_out_flag--;
 		}
	}

	//---------------------------------------------------------------------
	//	USBf[^ʐM
	if(!HIDRxHandleBusy(USBOutHandle))				//Check if data was received from the host.
	{
		// Mobt@NA
		for( fi = 0; fi < TX_BUFFER_SIZE; fi++)
		{
		    ToSendDataBuffer[fi] = 0;
		}

		switch(ReceivedDataBuffer[0])
		{
		 	// t@[EFÃo[WPCɕԂ
			case 0x56:
				//Echo back to the host PC the command we are fulfilling in the first byte.  In this case, the Get Pushbutton State command.
				ToSendDataBuffer[0] = 0x56;
				tmp = strlen(c_version);
				if( 0 < tmp && tmp <= (64-2) )
				{
					for( fi = 0; fi < tmp; fi++ )
					{
						ToSendDataBuffer[fi+1] = c_version[fi];
					}
					// ŌNULLݒ
					ToSendDataBuffer[fi+1] = 0x00;
				}
				else
				{
					//o[Wُ
					ToSendDataBuffer[1] = 0x00;
				}
				if(!HIDTxHandleBusy(USBInHandle))
				{
					USBInHandle = HIDTxPacket(HID_EP4,(BYTE*)&ToSendDataBuffer[0],64);
				}
				break;

			// ԊOo̓f[^USBoRPCM
			case 0x60:
				//Echo back to the host PC the command we are fulfilling in the first byte.  In this case, the Get Remocon Data command.
				ToSendDataBuffer[0] = 0x60;

				uc_out_code_type = CODE_TYPE_STANDARD;
				for(fi = 0; fi < OUTBUFFER_SIZE; fi++){
					uc_out_buff[fi] = ReceivedDataBuffer[fi+1];
				}
				break;
		}
		 //Re-arm the OUT endpoint for the next packet
		USBOutHandle = HIDRxPacket(HID_EP4,(BYTE*)&ReceivedDataBuffer,64);
	}
}




// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************
// The USB firmware stack will call the callback functions USBCBxxx() in response to certain USB related
// events.  For example, if the host PC is powering down, it will stop sending out Start of Frame (SOF)
// packets to your device.  In response to this, all USB devices are supposed to decrease their power
// consumption from the USB Vbus to <2.5mA each.  The USB module detects this condition (which according
// to the USB specifications is 3+ms of no bus activity/SOF packets) and then calls the USBCBSuspend()
// function.  You should modify these callback functions to take appropriate actions for each of these
// conditions.  For example, in the USBCBSuspend(), you may wish to add code that will decrease power
// consumption from Vbus to <2.5mA (such as by clock switching, turning off LEDs, putting the
// microcontroller to sleep, etc.).  Then, in the USBCBWakeFromSuspend() function, you may then wish to
// add code that undoes the power saving things done in the USBCBSuspend() function.

// The USBCBSendResume() function is special, in that the USB stack will not automatically call this
// function.  This function is meant to be called from the application firmware instead.  See the
// additional comments near the function.

/******************************************************************************
 * Function:        void USBCBSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Call back that is invoked when a USB suspend is detected
 *
 * Note:            None
 *****************************************************************************/
void USBCBSuspend(void)
{
	//Example power saving code.  Insert appropriate code here for the desired
	//application behavior.  If the microcontroller will be put to sleep, a
	//process similar to that shown below may be used:

	//ConfigureIOPinsForLowPower();
	//SaveStateOfAllInterruptEnableBits();
	//DisableAllInterruptEnableBits();
	//EnableOnlyTheInterruptsWhichWillBeUsedToWakeTheMicro();	//should enable at least USBActivityIF as a wake source
	//Sleep();
	//RestoreStateOfAllPreviouslySavedInterruptEnableBits();	//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.
	//RestoreIOPinsToNormal();									//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.

	//IMPORTANT NOTE: Do not clear the USBActivityIF (ACTVIF) bit here.  This bit is
	//cleared inside the usb_device.c file.  Clearing USBActivityIF here will cause
	//things to not work as intended.


	#if defined(__C30__)
	#if 0
		U1EIR = 0xFFFF;
		U1IR = 0xFFFF;
		U1OTGIR = 0xFFFF;
		IFS5bits.USB1IF = 0;
		IEC5bits.USB1IE = 1;
		U1OTGIEbits.ACTVIE = 1;
		U1OTGIRbits.ACTVIF = 1;
		Sleep();
	#endif
	#endif
}


/******************************************************************************
 * Function:        void _USB1Interrupt(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the USB interrupt bit is set
 *					In this example the interrupt is only used when the device
 *					goes to sleep when it receives a USB suspend command
 *
 * Note:            None
 *****************************************************************************/
#if 0
void __attribute__ ((interrupt)) _USB1Interrupt(void)
{
	#if !defined(self_powered)
		if(U1OTGIRbits.ACTVIF)
		{
			IEC5bits.USB1IE = 0;
			U1OTGIEbits.ACTVIE = 0;
			IFS5bits.USB1IF = 0;

			//USBClearInterruptFlag(USBActivityIFReg,USBActivityIFBitNum);
			USBClearInterruptFlag(USBIdleIFReg,USBIdleIFBitNum);
			//USBSuspendControl = 0;
		}
	#endif
}
#endif

/******************************************************************************
 * Function:        void USBCBWakeFromSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The host may put USB peripheral devices in low power
 *					suspend mode (by "sending" 3+ms of idle).  Once in suspend
 *					mode, the host may wake the device back up by sending non-
 *					idle state signalling.
 *
 *					This call back is invoked when a wakeup from USB suspend
 *					is detected.
 *
 * Note:            None
 *****************************************************************************/
void USBCBWakeFromSuspend(void)
{
	// If clock switching or other power savings measures were taken when
	// executing the USBCBSuspend() function, now would be a good time to
	// switch back to normal full power run mode conditions.  The host allows
	// a few milliseconds of wakeup time, after which the device must be
	// fully back to normal, and capable of receiving and processing USB
	// packets.  In order to do this, the USB module must receive proper
	// clocking (IE: 48MHz clock must be available to SIE for full speed USB
	// operation).
}

/********************************************************************
 * Function:        void USBCB_SOF_Handler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB host sends out a SOF packet to full-speed
 *                  devices every 1 ms. This interrupt may be useful
 *                  for isochronous pipes. End designers should
 *                  implement callback routine as necessary.
 *
 * Note:            None
 *******************************************************************/
void USBCB_SOF_Handler(void)
{
	// No need to clear UIRbits.SOFIF to 0 here.
	// Callback caller is already doing that.
}

/*******************************************************************
 * Function:        void USBCBErrorHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The purpose of this callback is mainly for
 *                  debugging during development. Check UEIR to see
 *                  which error causes the interrupt.
 *
 * Note:            None
 *******************************************************************/
void USBCBErrorHandler(void)
{
	// No need to clear UEIR to 0 here.
	// Callback caller is already doing that.

	// Typically, user firmware does not need to do anything special
	// if a USB error occurs.  For example, if the host sends an OUT
	// packet to your device, but the packet gets corrupted (ex:
	// because of a bad connection, or the user unplugs the
	// USB cable during the transmission) this will typically set
	// one or more USB error interrupt flags.  Nothing specific
	// needs to be done however, since the SIE will automatically
	// send a "NAK" packet to the host.  In response to this, the
	// host will normally retry to send the packet again, and no
	// data loss occurs.  The system will typically recover
	// automatically, without the need for application firmware
	// intervention.

	// Nevertheless, this callback function is provided, such as
	// for debugging purposes.
}


/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        When SETUP packets arrive from the host, some
 * 					firmware must process the request and respond
 *					appropriately to fulfill the request.  Some of
 *					the SETUP packets will be for standard
 *					USB "chapter 9" (as in, fulfilling chapter 9 of
 *					the official USB specifications) requests, while
 *					others may be specific to the USB device class
 *					that is being implemented.  For example, a HID
 *					class device needs to be able to respond to
 *					"GET REPORT" type of requests.  This
 *					is not a standard USB chapter 9 request, and
 *					therefore not handled by usb_device.c.  Instead
 *					this request should be handled by class specific
 *					firmware, such as that contained in usb_function_hid.c.
 *
 * Note:            None
 *******************************************************************/
void USBCBCheckOtherReq(void)
{
	USBCheckHIDRequest();
}//end


/*******************************************************************
 * Function:        void USBCBStdSetDscHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USBCBStdSetDscHandler() callback function is
 *					called when a SETUP, bRequest: SET_DESCRIPTOR request
 *					arrives.  Typically SET_DESCRIPTOR requests are
 *					not used in most applications, and it is
 *					optional to support this type of request.
 *
 * Note:            None
 *******************************************************************/
void USBCBStdSetDscHandler(void)
{
	// Must claim session ownership if supporting this request
}//end


/*******************************************************************
 * Function:        void USBCBInitEP(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the device becomes
 *                  initialized, which occurs after the host sends a
 * 					SET_CONFIGURATION (wValue not = 0) request.  This
 *					callback function should initialize the endpoints
 *					for the device's usage according to the current
 *					configuration.
 *
 * Note:            None
 *******************************************************************/
void USBCBInitEP(void)
{
	//enable the HID endpoint
	USBEnableEndpoint(HID_EP,USB_OUT_ENABLED|USB_IN_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
	USBEnableEndpoint(HID_EP2,USB_OUT_ENABLED|USB_IN_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
	USBEnableEndpoint(HID_EP3,USB_OUT_ENABLED|USB_IN_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
	USBEnableEndpoint(HID_EP4,USB_OUT_ENABLED|USB_IN_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
}

/********************************************************************
 * Function:        void USBCBSendResume(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB specifications allow some types of USB
 * 					peripheral devices to wake up a host PC (such
 *					as if it is in a low power suspend to RAM state).
 *					This can be a very useful feature in some
 *					USB applications, such as an Infrared remote
 *					control	receiver.  If a user presses the "power"
 *					button on a remote control, it is nice that the
 *					IR receiver can detect this signalling, and then
 *					send a USB "command" to the PC to wake up.
 *
 *					The USBCBSendResume() "callback" function is used
 *					to send this special USB signalling which wakes
 *					up the PC.  This function may be called by
 *					application firmware to wake up the PC.  This
 *					function should only be called when:
 *
 *					1.  The USB driver used on the host PC supports
 *						the remote wakeup capability.
 *					2.  The USB configuration descriptor indicates
 *						the device is remote wakeup capable in the
 *						bmAttributes field.
 *					3.  The USB host PC is currently sleeping,
 *						and has previously sent your device a SET
 *						FEATURE setup packet which "armed" the
 *						remote wakeup capability.
 *
 *					This callback should send a RESUME signal that
 *                  has the period of 1-15ms.
 *
 * Note:            Interrupt vs. Polling
 *                  -Primary clock
 *                  -Secondary clock ***** MAKE NOTES ABOUT THIS *******
 *                   > Can switch to primary first by calling USBCBWakeFromSuspend()

 *                  The modifiable section in this routine should be changed
 *                  to meet the application needs. Current implementation
 *                  temporary blocks other functions from executing for a
 *                  period of 1-13 ms depending on the core frequency.
 *
 *                  According to USB 2.0 specification section 7.1.7.7,
 *                  "The remote wakeup device must hold the resume signaling
 *                  for at lest 1 ms but for no more than 15 ms."
 *                  The idea here is to use a delay counter loop, using a
 *                  common value that would work over a wide range of core
 *                  frequencies.
 *                  That value selected is 1800. See table below:
 *                  ==========================================================
 *                  Core Freq(MHz)      MIP         RESUME Signal Period (ms)
 *                  ==========================================================
 *                      48              12          1.05
 *                       4              1           12.6
 *                  ==========================================================
 *                  * These timing could be incorrect when using code
 *                    optimization or extended instruction mode,
 *                    or when having other interrupts enabled.
 *                    Make sure to verify using the MPLAB SIM's Stopwatch
 *                    and verify the actual signal on an oscilloscope.
 *******************************************************************/
void USBCBSendResume(void)
{
	static WORD delay_count;

	USBResumeControl = 1;                // Start RESUME signaling

	delay_count = 1800U;                // Set RESUME line for 1-13 ms
	do
	{
		delay_count--;
	}while(delay_count);
	USBResumeControl = 0;
}


/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *                        USB_EVENT event, void *pdata, WORD size)
 *
 * PreCondition:    None
 *
 * Input:           USB_EVENT event - the type of event
 *                  void *pdata - pointer to the event data
 *                  WORD size - size of the event data
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called from the USB stack to
 *                  notify a user application that a USB event
 *                  occured.  This callback is in interrupt context
 *                  when the USB_INTERRUPT option is selected.
 *
 * Note:            None
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
	switch(event)
	{
		case EVENT_CONFIGURED:
			USBCBInitEP();
			break;
		case EVENT_SET_DESCRIPTOR:
			USBCBStdSetDscHandler();
			break;
		case EVENT_EP0_REQUEST:
			USBCBCheckOtherReq();
			break;
		case EVENT_SOF:
			USBCB_SOF_Handler();
			break;
		case EVENT_SUSPEND:
			USBCBSuspend();
			break;
		case EVENT_RESUME:
			USBCBWakeFromSuspend();
			break;
		case EVENT_BUS_ERROR:
			USBCBErrorHandler();
			break;
		case EVENT_TRANSFER:
			Nop();
			break;
		default:
			break;
	}
	return TRUE;
}

/*******************************************************************
 * Function:        int RemoconOutData(
 *                        void )
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          int - A processing result is returned. 0 is normal.
 *
 * Side Effects:    None
 *
 * Overview:        ̊֐́AԊOR̃f[^o͂܂B
 *                  o̓obt@[0]̃tH[}bg^Cv]A[1]ȍ~̃f[^o͂܂B
 *                  0 is returned at the time of normal processing.
 *
 * Note:            None
 *******************************************************************/
int RemoconOutData(void)
{
	int ret_code = 0;
	int fi = 0;
	unsigned int byte_pos = 0;
	unsigned int bit_pos = 0;
	unsigned char send_bit = 0;
	unsigned char format_type;
	unsigned char out_buffer_offset;

	/* o͏ԂłȂ */
	if( 0 >= ui_out_on_time && 0 >= ui_out_off_time )
	{
		/* o͑҂ */
		if( uc_out_status == OUTSTATUS_OUTPUT_WAIT )
		{
			/*  */
			ui_out_pos = 0;	/* o͈ʒu */

			// format type
			format_type = (uc_out_buff[0] & 0x0F);

			/* o̓obt@ɏo̓f[^H */
			if( format_type != FORMATCODE_UNKNOWN )
			{
				/* ̏Ԃ */
				uc_out_status = OUTSTATUS_DATA_CODE;

				/* f[^TCYݒ(bit) */
				// W^ 0oCgڂ̏4bit
				/* uc_out_buff[0]̏4rbgɏo̓f[^/4̒lĂ */
				/* (uc_out_buff[0] >> 4) * 4 = uc_out_buff[0] >> 2 */
				uc_send_bit_size = uc_out_buff[0];
				uc_send_bit_size = ((uc_send_bit_size&0xF0)>>2);

				// PWM Duty Set
				ClosePWM1();
				// 38kHz
				ui_PWM50_Set_Val = PWM_50;
				OpenPWM1(PWM_PERIOD);

				/* NECtH[}bg */
				if( format_type == FORMATCODE_NEC )
				{
					ui_out_on_time = READERCODE_ON_NEC;
					ui_out_off_time = READERCODE_OFF_NEC;

					uc_out_data0_on = DATA0_ON_NEC;
					uc_out_data0_off = DATA0_OFF_NEC;
					uc_out_data1_on = DATA1_ON_NEC;
					uc_out_data1_off = DATA1_OFF_NEC;
				}
				/* tH[}bgs */
				else
				{
					/* o̓obt@NA */
					for( fi = 0; fi < OUTBUFFER_SIZE_EX; fi++ )
					{
						uc_out_buff[fi] = 0;
					}
					uc_out_status = OUTSTATUS_OUTPUT_WAIT;
					uc_send_bit_size = 0;
					format_type = 0;
				}
			}
		}
		else if( uc_out_status == OUTSTATUS_DATA_CODE )
		{
			// tH[}bg^CvZbg
			format_type = (uc_out_buff[0] & 0x0F);
			out_buffer_offset = OUTBUFFER_SIZE - OUTBUFFER_DATA_SIZE;

			byte_pos = ui_out_pos / 8;
			bit_pos = ui_out_pos % 8;

			if ( ( ui_out_pos == (unsigned int)uc_send_bit_size ) || ui_out_pos > DATA_MAX_BITS )
			{	/* IR[h */
				send_bit = 0xF0;
			}
			else if( 0 <= (byte_pos+out_buffer_offset) && (byte_pos+out_buffer_offset) < OUTBUFFER_SIZE_EX )
			{
				// Bit瑗
				send_bit = (uc_out_buff[byte_pos+out_buffer_offset] >> (7-bit_pos) ) & 0x01;
			}
			else
			{	/* G[ */
				send_bit = 0xFF;
			}

			if( send_bit == 0 )
			{	/* DATA0o */
				ui_out_on_time = uc_out_data0_on;
				ui_out_off_time = uc_out_data0_off;
			}
			else if ( send_bit == 1 )
			{	/* DATA1o */
				ui_out_on_time = uc_out_data1_on;
				ui_out_off_time = uc_out_data1_off;
			}
			else if ( send_bit == 0xF0 )
			{	/* IR[ho */
				ui_out_on_time = uc_out_data0_on;
				ui_out_off_time = SENDCODE_END_CNT;
			}
			else if ( send_bit == 0xFF )
			{	/* G[ */
				ui_out_on_time = 0;
				ui_out_off_time = 0;
				uc_out_status = OUTSTATUS_OUTPUT_WAIT;
			}

			if( send_bit >= 0xF0 )
			{	/* ̑Mf[^Ȃ */
				/* o̓obt@NA */
				for( fi = 0; fi < OUTBUFFER_SIZE_EX; fi++ )
				{
					uc_out_buff[fi] = 0;
				}
				/* o͑҂ */
				uc_out_status = OUTSTATUS_OUTPUT_WAIT;
			}
			else if(send_bit == 0 || send_bit == 1)
			{
				/* o̓f[^ʒuCNg */
				ui_out_pos++;
			}
		}
	}

	/* oON */
	if( 0 < ui_out_on_time )
	{
		SetDCPWM1(ui_PWM50_Set_Val);
		//SetDCPWM1(PWM_50);
		ui_out_on_time--;
	}
	/* oOFF */
	else if( 0 < ui_out_off_time )
	{
		SetDCPWM1(PWM_OFF);
		ui_out_off_time--;
	}
	else
	{
		SetDCPWM1(PWM_OFF);
	}

	return ret_code;
}

#endif
